#define _LARGEFILE_SOURCE 
#define _LARGEFILE64_SOURCE
#define _FILE_OFFSET_BITS 64
#if defined(__linux)
/* ... and "engage" glibc large file support */
#define _GNU_SOURCE
#endif

#include <stdio.h>
#include <unistd.h>
#include <stdlib.h>
#include <sys/types.h>

#include "transport.hxx"

static int	media_written=0,next_track=1;
extern int	media_fd,dvd_compat;
extern char	*media_device;

extern "C"
off64_t get_capacity (int fd)
{ Scsi_Command cmd(fd);
  unsigned char buf[12];
  off64_t ret;
  struct stat64 sb;

    if (fstat64 (fd,&sb) || S_ISREG(sb.st_mode)) return 0;

    cmd[0] = 0x23;		// READ FORMAT CAPACITIES
    cmd[8] = sizeof(buf);
    cmd[9] = 0;
    if (cmd.transport (READ,buf,sizeof(buf)))
    {	perror (":-( unanble to READ FORMAT CAPACITIES");
	return 0;
    }

    ret  = buf[4]<<24;
    ret |= buf[5]<<16;
    ret |= buf[6]<<8;
    ret |= buf[7];
    ret *= buf[9]<<16|buf[10]<<8|buf[11];

  return ret;
}

extern "C"
ssize_t poor_mans_pwrite64 (int fd,const void *buff,size_t size,off64_t foff)
{ Scsi_Command cmd(fd);
  unsigned char *sense=cmd.sense();
  unsigned int lba;

    if (foff&2047 || size&2047)	/* 2K block size	*/
	return -1;

    lba = foff>>11;

    cmd[0] = 0x2A;		// WRITE(10)
    cmd[2] = (lba>>24)&0xff;	// Logical Block Addrss
    cmd[3] = (lba>>16)&0xff;
    cmd[4] = (lba>>8)&0xff;
    cmd[5] = lba&0xff;
    cmd[7] = (size>>19)&0xff;
    cmd[8] = (size>>11)&0xff;
    cmd[9] = 0;
    if (cmd.transport (WRITE,(void *)buff,size))
    {	fprintf (stderr,":-[ SENSE KEY=%xh/ASC=%xh/ASCQ=%xh ]\n",
			sense[2]&0x0F,sense[12],sense[13]);
	return -1;
    }
    media_written = 1;

  return size;
}

extern "C"
int plus_rw_restart_format (int fd)
{ Scsi_Command cmd(fd);
  unsigned char descr[12];

    memset (descr,0,sizeof(descr));
    descr[1]=0x02;		// "IMMED" flag
    descr[3]=0x08;		// "Descriptor Length" (LSB)
    descr[8]=0x98;		// "Format type" 0x26
    descr[11]=1;		// "Restart Format"

    cmd[0] = 0x04;		// FORMAT UNIT
    cmd[1] = 0x11;		// "FmtData" and "Format Code"
    cmd[5] = 0;
    if (cmd.transport(WRITE,descr,sizeof(descr)))
    {	perror (":-( unable to RESTART FORMAT");
	return 1;
    }

    wait_for_unit(cmd,NULL);

  return 0;
}

extern "C"
void plus_rw_finalize ()
{   if (media_written)
    { Scsi_Command cmd(media_fd);

	fprintf (stderr,"%s: flushing cache\n",media_device);
	cmd[0] = 0x35;		// FLUSH CACHE
	cmd[1] = 0x02;		// "IMMED"
	cmd[9] = 0;
	if (cmd.transport())
	    perror (":-( unable to FLUSH CACHE");

	if (wait_for_unit (cmd,NULL)) return;

	if (!dvd_compat)
	{   fprintf (stderr,"%s: stopping de-icing\n",media_device);
	    cmd[0] = 0x5B;	// CLOSE TRACK/SESSION
	    cmd[1] = 0x01;	// "IMMED"
	    cmd[2] = 0;		// "Stop De-Icing"
	    cmd[9] = 0;
	    if (cmd.transport())
		perror (":-( unable to CLOSE SESSION");

	    if (wait_for_unit (cmd,NULL)) return;
	}

	fprintf (stderr,"%s: writing lead-out\n",media_device);
	cmd[0] = 0x5B;		// CLOSE TRACK/SESSION
	cmd[1] = 0x01;		// "IMMED"
	cmd[2] = 0x02;		// "Close session"
	cmd[9] = 0;
	if (cmd.transport())
	    perror (":-( unable to CLOSE SESSION");

	if (wait_for_unit (cmd,NULL)) return;

	media_written = 0;
    }
    _exit (errno);
}

extern "C"
void plus_r_finalize ()
{   if (media_written)
    { Scsi_Command cmd(media_fd);
      int mode;

	fprintf (stderr,"%s: flushing cache\n",media_device);
	cmd[0] = 0x35;		// FLUSH CACHE
	cmd[9] = 0;
	if (cmd.transport())
	    perror (":-( unable to FLUSH CACHE");

	if (wait_for_unit (cmd,NULL)) return;

	fprintf (stderr,"%s: closing track\n",media_device);
	cmd[0] = 0x5B;		// CLOSE TRACK/SESSION
	cmd[1] = 0x01;		// "IMMED"
	cmd[2] = 0x01;		// "Close Track"
	cmd[4] = next_track>>8;
	cmd[5] = next_track;
	cmd[9] = 0;
	if (cmd.transport())
	    perror (":-( unable to CLOSE SESSION");

	if (wait_for_unit (cmd,NULL)) return;

	if (dvd_compat)
	    mode=0x06, fprintf (stderr,"%s: closing disk\n",media_device);
	else
	    mode=0x02, fprintf (stderr,"%s: closing session\n",media_device);

	cmd[0] = 0x5B;		// CLOSE TRACK/SESSION
	cmd[1] = 0x01;		// "IMMED"
	cmd[2] = mode;		// "Close session"
	cmd[9] = 0;
	if (cmd.transport())
	    perror (":-( unable to CLOSE SESSION");

	if (wait_for_unit (cmd,NULL)) return;

	media_written = 0;
    }
    _exit (errno);
}

extern "C"
int get_mmc_profile (int fd)
{ Scsi_Command cmd(fd);
  unsigned char buf[8];

    cmd[0] = 0x46;
    cmd[8] = sizeof(buf);
    cmd[9] = 0;
    if (cmd.transport(READ,buf,sizeof(buf)))
	perror (":-( unable to GET CONFIGURATION"),
	exit (1);

  return buf[6]<<8|buf[7];
}

extern "C"
int plus_r_C_parm (int fd,char *C_parm)
{ unsigned int next_session, prev_session;
  Scsi_Command cmd(fd);
  unsigned char *sense=cmd.sense(), buf[36];

    cmd[0] = 0x52;		// GET TRACK INFORMATION
    cmd[1] = 1;
    cmd[5] = 0xFF;		// ask for "invisible track"
    cmd[8] = sizeof(buf);
    cmd[9] = 0;
    if (cmd.transport (READ,buf,sizeof(buf)))
    {	perror (":-( unable to GET TRACK INFORMATION");
	if (sense[12]==0x24) // hp dvd200i returns 24 if media is full
	    fprintf (stderr,":-( media must be full already\n");
	exit (1);
    }
    next_session  = buf[12]<<24;
    next_session |= buf[13]<<16;
    next_session |= buf[14]<<8;
    next_session |= buf[15];

    next_track = buf[2]|buf[32]<<8;

    cmd[0] = 0x43;		// READ TOC
    cmd[2] = 1;			// "Session info"
    cmd[8] = 12;
    cmd[9] = 0;
    if (cmd.transport (READ,buf,12))
	perror (":-( unable to READ TOC"),
	exit (1);

    prev_session  = buf[8]<<24;
    prev_session |= buf[9]<<16;
    prev_session |= buf[10]<<8;
    prev_session |= buf[11];

    sprintf (C_parm,"%d,%d",prev_session+16,next_session);

  return next_session;
}

extern "C"
int flush_cache (int fd)
{ Scsi_Command cmd(fd);

    cmd[0] = 0x35;		// FLUSH CACHE
    cmd[9] = 0;

  return cmd.transport();
}
